"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import pygame
 
from resourcemanager import ResourceManager
from menuitems import MenuItem, MenuItemSelector
from game import Game
from level import Level, LevelFactory
from tower import TowerMenu
from levelmenu import LevelMenu
from highscore import HighScore

from menu import Menu

class GameMenu(Menu):
	def __init__(self, screen, runner):
		Menu.__init__(self, screen, runner)
		
		self.endlessmenu   = LevelMenu(screen, runner, self, True)		
		self.skirmishmenu  = LevelMenu(screen, runner, self, False)
		self.highscoremenu = HighscoreMenu(screen, runner, self)
		self.helpmenu      = HelpMenu(screen, runner, self)
	
	def show_multiplayer_menu(self):
		self.enter_child(self.multiplayermenu)
	
	def start_campaign(self):
		g = Game(["level_01", "level_02", "level_03", "level_04", "level_05"], TowerMenu(self.screen), self.screen)
		g.run()	
	
	def show_skirmish_menu(self):
		self.skirmishmenu.run()
	
	def show_endless_menu(self):
		self.endlessmenu.run()
	
	def show_highscore_menu(self):
		self.highscoremenu.run()
	
	def show_help_menu(self):
		self.helpmenu.run()
	
	def initiate_menu_items(self):
		item = MenuItem("campaign.png", "campaign_off.png", True)
		item.add_action(self.start_campaign)
		self.add_menu_item(item)
		
		item = MenuItem("skirmish.png", "skirmish_off.png", True)
		item.add_action(self.show_skirmish_menu)
		self.add_menu_item(item)
		
		item = MenuItem("endless.png", "endless_off.png",   True)
		item.add_action(self.show_endless_menu)
		self.add_menu_item(item)
		
		item = MenuItem("highscore.png", "highscore_off.png", True)
		item.add_action(self.show_highscore_menu)
		self.add_menu_item(item)
		
		item = MenuItem("help.png", "help_off.png", True)
		item.add_action(self.show_help_menu)
		self.add_menu_item(item)
		
		item = MenuItem("exit.png", "exit_off.png", True)
		item.add_action(self.back_to_parent)
		self.add_menu_item(item)

class HighscoreMenu(Menu):
	def __init__(self, screen, runner, parent):
		Menu.__init__(self, screen, runner, parent)
		
		self.highscoreView = HighscoreView(screen, runner, self)
	
	def initiate_menu_items(self):
		self.add_menu_item(MenuItem("highscore_off.png", "highscore_off.png", False))
		
		item = MenuItem("campaign.png", "campaign_off.png", True)
		item.add_action(self.show_highscore_campaign)
		self.add_menu_item(item)
		item = MenuItem("skirmish.png", "skirmish_off.png", True)
		item.add_action(self.show_highscore_skirmish)
		self.add_menu_item(item)
		item = MenuItem("endless.png", "endless_off.png", True)
		item.add_action(self.show_highscore_endless)
		self.add_menu_item(item)
		
		item = MenuItem("back.png", "back_off.png", True)
		item.add_action(self.back_to_parent)
		self.add_menu_item(item)


	def show_highscore_campaign(self):
		self.highscoreView.set_Mode(3)
		self.highscoreView.run()
		
	def show_highscore_skirmish(self):
		self.highscoreView.set_Mode(2)
		self.highscoreView.run()
		
	def show_highscore_endless(self):
		self.highscoreView.set_Mode(1)
		self.highscoreView.run()

class HighscoreView(Menu):
	def __init__(self, screen, runner, parent):
		Menu.__init__(self, screen, runner, parent)
		
		self.fontHead = ResourceManager.get_font(24)
		self.font = ResourceManager.get_font(18)
		self.fontSurface = None
		self.mode = 1
	
	def initiate_menu_items(self):
		self.add_menu_item(MenuItem("highscore_off.png", "highscore_off.png", False))
		item = MenuItem("back.png", "back_off.png", True)
		item.add_action(self.back_to_parent)
		self.add_menu_item(item)

	def set_Mode(self, mode):
		self.mode = mode
	
	def show_Highscore(self):
		h = HighScore()
		pos = self.calculate_positions(7)
		scores = h.read_highscores_from_database(self.mode)
		
		menuheight = self.menuzone[1] - self.menuzone[0]
		posY = self.menuzone[0] + 2 * self.itemSize[1] + 10

		height = pos[5][1] - pos[0][1]
		
		self.menuItems[0].set_position(pos[0], False)
		self.menuItems[1].set_position(pos[6], False)
		self.items = pygame.sprite.RenderPlain(self.menuItems)
		
		self.fontSurface = self.fontHead.render(str("Rank"	), True, pygame.Color(0,0,0))
		self.screen.blit(self.fontSurface, (pos[1][0]+10, pos[1][1]+10))
		self.fontSurface = self.fontHead.render(str("Player"	), True, pygame.Color(0,0,0))
		self.screen.blit(self.fontSurface, (pos[1][0]+100, pos[1][1]+10))
		self.fontSurface = self.fontHead.render(str("Score"	), True, pygame.Color(0,0,0))
		self.screen.blit(self.fontSurface, (pos[1][0] + self.itemSize[0]-100, pos[1][1]+10))
		
		padding = 20;
		posy = pos[1][1]+10 + 50
		for idx in xrange(0, len(scores)):
			self.fontSurface = self.font.render(str(idx+1), True, pygame.Color(0,0,0))
			self.screen.blit(self.fontSurface, (pos[1][0]+10, posy))
			self.fontSurface = self.font.render(str(scores[idx][0]	), True, pygame.Color(0,0,0))
			self.screen.blit(self.fontSurface, (pos[1][0]+100, posy))
			self.fontSurface = self.font.render(str(scores[idx][1]	), True, pygame.Color(0,0,0))
			self.screen.blit(self.fontSurface, (pos[1][0] + self.itemSize[0]-100, posy))
			posy += padding
		
		#self.item.gg
		#print scores
		
	def calculate_scorePosition():
		menuheight = self.menuzone[1] - self.menuzone[0]
		posY = self.menuzone[0] + 2 * self.itemSize[0] + 10
		posX = 150
		
		hieght = self.menuzone[1] - 10 -posY
		pygame.draw.rect(self.surface, (0,0,0,0), pygame.Rect(posX, posY, 300, height))
	
	def execution_loop(self, tick):
		self.handle_events()
		self.items.clear(self.screen, self.bgImage)
		self.animations.update(tick)
		self.items.update(tick)
		
		self.animations.clear(self.screen, self.bgImage)
		self.animations.draw(self.screen)
		self.show_Highscore()
		self.items.draw(self.screen)
	
class HelpMenu(Menu):
	def __init__(self, screen, runner, parent):
		Menu.__init__(self, screen, runner, parent)
	
	def initiate_menu_items(self):
		self.add_menu_item(MenuItem("help_off.png", "help_off.png", False))
		
		item = MenuItem("back.png", "back_off.png", True)
		item.add_action(self.back_to_parent)
		self.add_menu_item(item)