"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
from vector2D import Vector2D
from gesture import Gesture
import sqlite3.dbapi2 as db
import os

class HighScore(object):
	"""This class is used for gesture detection."""
	
	def __init__(self):
		"""Initiates the GestureDetector."""
		database = os.path.join(os.path.dirname(os.path.realpath(__file__)), "..", "data", "highscores", "highscores.db")
		self.connection = self.get_database_connection(database)
		#self.execute_sql("CREATE DATABASE highscores")
		
	def createTableAndDefaultEntries(self):
		self.execute_sql("DROP TABLE highscores")
		self.execute_sql("CREATE TABLE highscores ('Player' 'char', 'Score' bigint, 'Mode' integer)")
		#Modes: 1 - endless; 2 - skirmish; 3 - campaign
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('FH', 900, 1)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('FH', 1500, 1)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('FH', 2000, 1)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('CS', 900, 2)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('CS', 1500, 2)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('CS', 2000, 2)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('MK', 900, 3)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('MK', 1500, 3)")
		self.execute_sql("INSERT INTO highscores (Player, Score, Mode) VALUES ('MK', 2000, 3)")

	def execute_sql(self, statement, returnResult=False):
		"""Executes the given SQL statement.
		
		The database connection has to be stored in self.connection.
		
		@param statement Can be a single SQL statement or a list of statements
						 that should be executed in order.
		@param returnResult If this is True the result of the SQL statement is
		                    returned.
		
		@return If returnResult is True, the result of the last statement is
		        returned, None otherwise.
		"""
		cursor = self.connection.cursor()
		
		if isinstance(statement, basestring):
			cursor.execute(statement)
		elif isinstance(statement, list):
			for sql in statement:
				cursor.execute(sql)
		
		self.connection.commit()
		
		result = None
		if returnResult:
			result = cursor.fetchall()
		
		cursor.close()
		return result

	def get_database_connection(self, database):
		"""Opens a connection to the gesture database.
		
		@param database Database connection string.
		
		@return returns a connection to the database.
		"""
		if not os.path.exists(database):
			raise SystemExit("Cannot find gesture database: " + str(database))
		
		return db.connect(database)
	
	def read_highscores_from_database(self, mode):
		"""Reads the gestures from the database.
		
		The database connection has to be stored in self.connection.
		self.detaillevel has to be set!
		
		@return Returns a list containing the finalized gestures from the database.
		        The index of each gestures represents their ids in the database.
		"""
		sql = "SELECT * FROM highscores WHERE mode = " + str(mode) + " ORDER BY score DESC LIMIT 0,10;"
		dbHighScores = self.execute_sql(sql, True)
		
		res = []
		for score in dbHighScores:
			
			res.append(score)		
		return res
	
	def write_highscores_to_database(self, player, score, mode):
		"""Reads the gestures from the database.
		
		The database connection has to be stored in self.connection.
		self.detaillevel has to be set!
		
		@return Returns a list containing the finalized gestures from the database.
		        The index of each gestures represents their ids in the database.
		"""
		sql = "INSERT INTO highscores (Player, Score, Mode) VALUES ('" + player+ "', " + str(score) + ", " + str(mode) + " )"
		dbHighScores = self.execute_sql(sql, True)
	
if __name__ == "__main__":
	score = HighScore()
	#score.createTableAndDefaultEntries()
	print score.read_highscores_from_database(3)
	