"""
Copyright (c) 2009 [name + "(at)student.hpi.uni-potsdam.de" for name in ["murat.knecht", "franziska.haeger", "christian.schwarz"]]

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
"""
import threading, time, mutex, astar

from vector2D import Vector2D

class Path(object):
	"""This class represents the path."""
	id = 0
	
	def __init__(self, route):
		self.id = Path.id
		Path.id += 1
		self.tiles = []
		
		for tile in route:
			if not isinstance(tile, Vector2D):
				tile = Vector2D(tile)
			self.tiles.append(tile)
	
	def __eq__(self, other):
		"""Compares the two path instances."""
		if not isinstance(other, Path):
			return False
		
		return self.id == other.id
	
	def get_next_tile(self, lastiter=0):
		"""Returns the next tile, which is part of the path.
		
		@param lastiter The current tile of the requester.
		
		@return Returns the next tile of the path. If the current tile
		        is the target, None will be returned.
		"""
		if lastiter == len(self.tiles) - 1:
			return lastiter, self.tiles[-1]
		
		return lastiter+1, self.tiles[lastiter+1]
	
	def get_previous_tile(self, lastiter=0):
		"""Returns the next tile, which is part of the path.
		
		@param lastiter The current tile of the requester.
		
		@return Returns the next tile of the path. If the current tile
		        is the target, None will be returned.
		"""
		if lastiter == 0:
			return lastiter, self.tiles[0]
		
		return lastiter-1, self.tiles[lastiter-1]


	def get_start_pos(self):
		return self.tiles[0]
		
	
	def get_current_iter_on_path(self, currentpos):
		"""Returns the current iterator for the path.
		
		@todo beautify this function!!! T-002
		"""
		if not isinstance(currentpos, Vector2D):
			currentpos = Vector2D(currentpos)
		
		res = 0
		mindist = -1
		for idx in xrange(len(self.tiles)):
			dist = (currentpos - self.tiles[idx]).magnitude()
			
			if mindist == -1 or dist <= mindist:
				mindist = dist
				res = idx
		
		return res

class Pathfinder(threading.Thread):
	
	def __init__(self):
		threading.Thread.__init__(self)
		self.mutex = mutex.mutex()
		self.level = None
		self.shouldRun = True
		self.path = {}
		self.airpath = {}
	
	def __del__(self):
		print "Pathfinder was deleted!!!"
		self.shouldRun = False
	#	self.join()
	
	def acquire_mutex(self):
		while not self.mutex.testandset():
			print "levelmutex was locked."
			time.sleep(0.01)
	
	def release_mutex(self):
		self.mutex.unlock()
	
	def set_level(self, level):
		self.acquire_mutex()
		self.level = level
		self.release_mutex()
	
	def get_and_and_reset_level(self):
		self.acquire_mutex()
		level = self.level
		self.level = None
		self.release_mutex()		
		return level
	
	def get_airpath(self, way):
		self.acquire_mutex()
		if self.airpath.has_key(way):
			res = self.airpath[way]
		else:
			res = None
		self.release_mutex()
		
		return res


	def get_path(self, way):
		self.acquire_mutex()
		if self.path.has_key(way):
			res = self.path[way]
		else:
			res = None
		self.release_mutex()
		
		return res

	def run(self):
		while self.shouldRun:
			while not self.level:
				time.sleep(0.1)
			level = self.get_and_and_reset_level()
			
			astar.max_x = level.size.x
			astar.max_y= level.size.y
			for start in level.startpoints:
				res = astar.findpath(start, level.endpoint, level.moveables)
				self.acquire_mutex()
				self.path[(level.endpoint, start)] = Path(res)
				self.release_mutex()
				res = astar.findpath(start, level.endpoint, level.mapTiles)
				self.acquire_mutex()
				self.airpath[(level.endpoint, start)] = Path(res)
				self.release_mutex()
			if len(level.wayTiles) > 0:
				for middle in level.wayTiles:
					for start in level.startpoints:
						res = astar.findpath(start, middle, level.moveables)
						res2 = astar.findpath(middle, level.endpoint, level.moveables)
						res2.remove(res2[0])
						for tile in res2:
							res.append(tile)
						self.acquire_mutex()
						self.path[(middle, start)] = Path(res)
						self.release_mutex()
						
						res = astar.findpath(start, middle, level.mapTiles)
						res2 = astar.findpath(middle, level.endpoint, level.mapTiles)
						res2.remove(res2[0])
						for tile in res2:
							res.append(tile)
						self.acquire_mutex()
						self.airpath[(middle, start)] = Path(res)
						self.release_mutex()